/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "OSS.h"
#include <errno.h>

static char* get_mixer_device(void)
{
	char *name;
	
	if (oss_cfg.use_alt_mixer_device && oss_cfg.alt_mixer_device)
		name = g_strdup(oss_cfg.alt_mixer_device);
	else if (oss_cfg.mixer_device > 0)
		name = g_strdup_printf("%s%d", DEV_MIXER, oss_cfg.mixer_device);
	else
		name = g_strdup(DEV_MIXER);

	return name;
}

#include <sys/io.h>

static int mixer_reg = 0, mixer_data = 0;

void oss_get_volume(int *l, int *r)
{
	int fd, v, cmd, devs;
	gchar *devname;

	devname = get_mixer_device();
	fd = open(devname, O_RDONLY);
	g_free(devname);

	/*
	 * We dont show any errors if this fails, as this is called
	 * rather often
	 */
	if (fd != -1)
	{
		ioctl(fd, SOUND_MIXER_READ_DEVMASK, &devs);
		if ((devs & SOUND_MASK_PCM) && (oss_cfg.use_master==0))
			cmd = SOUND_MIXER_READ_PCM;
		else if ((devs & SOUND_MASK_VOLUME) && (oss_cfg.use_master==1))
			cmd = SOUND_MIXER_READ_VOLUME;
		else
		{
			close(fd);
			return;
		}
		ioctl(fd, cmd, &v);
		*r = (v & 0xFF00) >> 8;
		*l = (v & 0x00FF);
		close(fd);
	}
}

int oss_set_3dse(int stat)
{
	int fd;
	gchar *devname;

	if ((stat != 0) && (stat != 1))
		return -1;

	if (oss_cfg.mixer_device > 0)
		devname = g_strdup_printf("/dev/mixer%d", oss_cfg.mixer_device);
	else
		devname = g_strdup("/dev/mixer");
	fd = open(devname, O_RDONLY);
	g_free(devname);

	if (fd != -1)
	{
		if (ioctl(fd, SOUND_MIXER_3DSE, &stat) < 0)
		{
			close(fd);
			return -1;
		}
		else
		{
			close(fd);
			return stat;
		}
	}
	return -1;
}

int oss_get_3dse(void) 
{
	int fd, test_stat;
	gchar *devname;

	if (oss_cfg.mixer_device > 0)
		devname = g_strdup_printf("/dev/mixer%d", oss_cfg.mixer_device);
	else
		devname = g_strdup("/dev/mixer");
	fd = open(devname, O_RDONLY);
	g_free(devname);

	if (fd != -1)
	{
		test_stat = 2; /* code 2 will request status */
		if (ioctl(fd, SOUND_MIXER_3DSE, &test_stat) < 0)
		{
			close(fd);
			return -1;
		}
		else
		{
			close(fd);
			return test_stat;
		}
	}
	return -1;	
}

static void check_legal_mixer ()
{
	if (!mixer_reg || !mixer_data)
	{
		fprintf(stderr, "Illegal mixer settings...\n");
		exit(1);
	}
}

void oss_hw_set_3dse(int stat)
{
	unsigned char b;
	check_legal_mixer ();
	switch(stat)
	{
	case 1:
		outb(AWE_3DSE, mixer_reg);
		b = inb(mixer_data);
		b |= 1;
		outb(b, mixer_data);
		break;
	case 0:
		outb(AWE_3DSE, mixer_reg);
		b = inb(mixer_data);
		b &= 0xff - 1;
		outb(b, mixer_data);
		break;
	}
}

int oss_hw_get_3dse(void) 
{
	unsigned char b;
	check_legal_mixer ();
	outb(AWE_3DSE, mixer_reg);
	b = inb(mixer_data);
	return (b & 1);
}

int initHardwareAccess () 
{
	char *blasterEnv, *Aptr, *Mptr;
	int tmp, mixer_base = 0;

	if ((blasterEnv = getenv ("BLASTER"))) 
	{
		Aptr = blasterEnv;
		while ((Aptr = strchr(Aptr, 'A')))
			if ((tmp = strtol(++Aptr, &Aptr, 16)))
				mixer_base = tmp;
		Mptr = blasterEnv;
		while ((Mptr = strchr(Mptr, 'M')))
			if ((tmp = strtol(++Mptr, &Mptr, 16)))
				mixer_base = tmp;
	}
	if (!mixer_base) 
	{
		mixer_base = MIXER_BASE_DEFAULT;
	}
	else 
	{
		switch (mixer_base)
		{
		case 0x210:
		case 0x220:
		case 0x230:
		case 0x240:
		case 0x250:
		case 0x260:
		case 0x280:
			break;
		default:
			fprintf (stderr, "Illegal mixer base.\n");
			exit (1);
		}
	}
	mixer_reg = mixer_base + MIXER_REG_OFFSET;
	mixer_data = mixer_base + MIXER_DATA_OFFSET;
	if (getuid () == 0 && geteuid () == 0)
	{
		ioperm (mixer_reg, 1, 1);
		ioperm (mixer_data, 1, 1);
		return 0;
	}
	else
	{
		return -1;
 	}
}

void oss_set_volume(int l, int r)
{
	int fd, v, cmd, devs;
	gchar *devname;

	devname = get_mixer_device();
	fd = open(devname, O_RDONLY);

	if (fd != -1)
	{
		ioctl(fd, SOUND_MIXER_READ_DEVMASK, &devs);
		if ((devs & SOUND_MASK_PCM) && (oss_cfg.use_master==0))
			cmd = SOUND_MIXER_WRITE_PCM;
		else if ((devs & SOUND_MASK_VOLUME) && (oss_cfg.use_master==1))
			cmd = SOUND_MIXER_WRITE_VOLUME;
		else
		{
			close(fd);
			return;
		}
		v = (r << 8) | l;
		ioctl(fd, cmd, &v);
		close(fd);
	}
	else
		g_warning("oss_set_volume(): Failed to open mixer device (%s): %s",
			  devname, strerror(errno));
	g_free(devname);
}
