#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <xmms/plugin.h>
#include <setjmp.h>
#include "st.h"
#include "sox.h"

static void sox_init(void);
static void sox_cleanup(void);
static int sox_mod_samples(gpointer * d, gint length, AFormat afmt, gint srate, gint nch);
extern void sox_configure(void);
static void sox_about(void);

#define MAX_SRATE 50000
#define MAX_CHANNELS 2
#define BYTES_PS 2

EffectPlugin sox_ep =
{
	NULL,
	NULL,
	"SOX Effects",
	sox_init,
	sox_cleanup,
	sox_about,	/* about */
	sox_configure,	/* configure */
	sox_mod_samples,
	NULL
};

EffectPlugin *get_eplugin_info(void)
{
	return &sox_ep;
}

struct effect eff[2];

#define MAXREVERBS 8
typedef struct reverbstuff {
        int     counter;
        int     numdelays;
        float   *reverbbuf;
        float   in_gain, out_gain, time;
        float   delay[MAXREVERBS], decay[MAXREVERBS];
        long    samples[MAXREVERBS], maxsamples;
        LONG    pl, ppl, pppl;
} *reverb_t;


jmp_buf sox_very_bad_hack;

static int started = 0;
int sox_effect_failed = 0;
int sox_restart_effect = 0;

static void sox_init_effect(void) {
	if (started && !sox_effect_failed) {
		eff[0].h->stop(&eff[0]);
		eff[1].h->stop(&eff[1]);
	}
	started = 0;

	eff[0].name = sox_configs[sox_cfg.effect_number].name;
	eff[1] = eff[0];

	sox_restart_effect = 0;
}

static void sox_init(void) {
	sox_read_config();

	eff[0].ininfo.rate = 0;
	eff[0].ininfo.channels = 0;
	eff[0].outinfo = eff[0].ininfo;

	started = 0;
	sox_restart_effect = 1;
}

static void sox_cleanup(void) {
	if (started && !sox_effect_failed) {
		eff[0].h->stop(&eff[0]);
		eff[1].h->stop(&eff[1]);
	}
	started = 0;
}

static void sox_to_S32(gint32 *ibuf, guchar *data, gint length, gint sample_sz, gint sample_signed, gint sample_wrong_endian, gint nch, gint channel) {
	gint sample_mul = 1<<((4-sample_sz)*8-1), i;
	guchar tmp;

	for (i = channel*sample_sz; i < length; i += sample_sz*nch) {
		if (sample_wrong_endian) {
			tmp = data[i+1];
			data[i+1] = data[i];
			data[i] = tmp;
		}

		switch (sample_sz) {
			case 1:
				ibuf[i/nch] = *(gchar*)(data+i);
				*(gchar*)(data+i) = 0;
				if (!sample_signed)
ibuf[i/nch] = ((guint32)ibuf[i/nch])-(1<<(sample_sz*8-1));
				break;
			case 2:
				ibuf[i/2/nch] = *(gint16*)(data+i);
				*(gint16*)(data+i) = 0;
				if (!sample_signed)
ibuf[i/2/nch] = ((guint32)ibuf[i/2/nch])-(1<<(sample_sz*16-1));
				break;
		}

		ibuf[i/sample_sz/nch] *= sample_mul;
	}
}

static void sox_S32_to(gint32 *obuf, guchar *data, gint length, gint sample_sz, gint sample_signed, gint sample_wrong_endian, gint nch, gint channel) {
	gint sample_mul = 1<<((4-sample_sz)*8-1), i;
	guchar tmp;

	for (i = channel*sample_sz; i < length; i += sample_sz*nch) {
		obuf[i/sample_sz/nch] /= sample_mul;

		switch (sample_sz) {
			case 1:
				if (!sample_signed)
obuf[i/nch] = obuf[i/nch]+(1<<(sample_sz*8-1));
				*(gchar*)(data+i) = obuf[i/nch];
				break;
			case 2:
				if (!sample_signed)
obuf[i/2/nch] = obuf[i/2/nch]+(1<<(sample_sz*16-1));
				*(gint16*)(data+i) = obuf[i/2/nch];
				break;
		}

		if (sample_wrong_endian) {
			tmp = data[i+1];
			data[i+1] = data[i];
			data[i] = tmp;
		}
	}
}

static int sox_mod_samples(gpointer* d, gint length, AFormat afmt, gint srate, gint nch)
{
	static guint32 *ibuf = NULL, *obuf = NULL;
	guchar *data = (guchar*)*d;

	gint sample_sz, sample_signed, sample_mul, sample_wrong_endian, sox_fmt;
	gint j, inlen, outlen;

	char **argv;
	int argc;

	if (sox_restart_effect) sox_init_effect();
	if (sox_effect_failed) return length;

	switch (afmt) {
		case FMT_U8:
			sample_mul = 1<<23;
			sample_signed = 0;
			sample_sz = 1;
			sample_wrong_endian = 0;
			break;
		case FMT_U16_NE:
			sample_mul = 1<<15;
			sample_signed = 0;
			sample_sz = 2;
			sample_wrong_endian = 0;
			break;
		case FMT_U16_LE:
			sample_mul = 1<<15;
			sample_signed = 0;
			sample_sz = 2;
			sample_wrong_endian = (G_BYTE_ORDER==G_LITTLE_ENDIAN)?0:1;
			break;
		case FMT_U16_BE:
			sample_mul = 1<<15;
			sample_signed = 0;
			sample_sz = 2;
			sample_wrong_endian = (G_BYTE_ORDER==G_LITTLE_ENDIAN)?1:0;
			break;

		case FMT_S8:
			sample_mul = 1<<23;
			sample_signed = 1;
			sample_sz = 1;
			sample_wrong_endian = 0;
			break;
		case FMT_S16_NE:
			sample_mul = 1<<15;
			sample_signed = 1;
			sample_sz = 2;
			sample_wrong_endian = 0;
			break;
		case FMT_S16_LE:
			sample_mul = 1<<15;
			sample_signed = 1;
			sample_sz = 2;
			sample_wrong_endian = (G_BYTE_ORDER==G_LITTLE_ENDIAN)?0:1;
			break;
		case FMT_S16_BE:
			sample_mul = 1<<15;
			sample_signed = 1;
			sample_sz = 2;
			sample_wrong_endian = (G_BYTE_ORDER==G_LITTLE_ENDIAN)?1:0;
	}

	if ((length != eff[0].ininfo.size) ||
	    (srate != eff[0].ininfo.rate) ||
	    (eff[0].ininfo.channels != nch) ||
	    !obuf || !ibuf || !started) {

		if (ibuf) { g_free(ibuf); ibuf = NULL; }
		if (obuf) { g_free(obuf); obuf = NULL; }

		eff[0].ininfo.size = length;
		if (eff[0].ininfo.rate != srate || eff[0].ininfo.channels != nch || !started) {
			if (started && !sox_effect_failed) {
				eff[0].h->stop(&eff[0]);
				eff[1].h->stop(&eff[1]);
			}
			started = 0;

			eff[1].ininfo.rate = eff[0].ininfo.rate = srate;
			eff[1].ininfo.channels = eff[0].ininfo.channels = nch;

			sox_effect_failed = 0;

			if (setjmp(sox_very_bad_hack)) {
				sox_effect_failed = 1;
				return length;
			}

			geteffect(&eff[0]);
			geteffect(&eff[1]);
			argc = sox_configs[sox_cfg.effect_number].build_opts(&argv);
			eff[0].h->getopts(&eff[0], argc, argv);
			eff[1].h->getopts(&eff[1], argc, argv);

			eff[0].h->start(&eff[0]);
			eff[1].h->start(&eff[1]);
			started = 1;
		}

		if (eff[0].h->flags&EFF_MCHAN) {
			ibuf = g_malloc((length/sample_sz)*4);
			obuf = g_malloc((length/sample_sz)*4);
		} else {
			ibuf = g_malloc((length/sample_sz)*4/nch);
			obuf = g_malloc((length/sample_sz)*4/nch);
		}
	}

	if (eff[0].h->flags&EFF_MCHAN) {
		sox_to_S32(ibuf, data, length, sample_sz, sample_signed, sample_wrong_endian, 1, 0);

		inlen = length / sample_sz;
		outlen = inlen;
		eff[0].h->flow(&eff[0], ibuf, obuf, &inlen, &outlen);
		outlen = outlen * sample_sz;

		sox_S32_to(obuf, data, length, sample_sz, sample_signed, sample_wrong_endian, 1, 0);
	} else {
		for (j = 0; j < nch; j++) {
			sox_to_S32(ibuf, data, length, sample_sz, sample_signed, sample_wrong_endian, nch, j);

			inlen = length / sample_sz / nch;
			outlen = inlen;
			eff[j].h->flow(&eff[j], ibuf, obuf, &inlen, &outlen);
			outlen = outlen * sample_sz * nch;

			sox_S32_to(obuf, data, length, sample_sz, sample_signed, sample_wrong_endian, nch, j);
		}
	}

	return outlen;
}

static void sox_about(void) {
	GtkWidget *dialog, *button, *label;

	dialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(dialog), "About SOX Effects "VERSION);
	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
	label = gtk_label_new(
"SOX Effects Plugin for XMMS\n\
by Zinx Verituse <zinx@linuxfreak.com>\n\
Copyright (C) 1999 Zinx Verituse\n\
\n\
This program is free software; you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation; either version 2 of the License, or\n\
(at your option) any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program; if not, write to the Free Software\n\
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307  USA\n\
\n\
NOTE: SOX 12.16 is under a different copyright; see the sox source files for\n\
more information (reverb.c is one, sox.c is not)"
);

	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	button = gtk_button_new_with_label(" Close ");
	gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dialog));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, FALSE, FALSE, 0);
	gtk_widget_show(button);

	gtk_widget_show(dialog);
	gtk_widget_grab_focus(button);
}
