
#include <gtk/gtk.h>
#include "libxmms/configfile.h"
#include "sox.h"

extern int sox_restart_effect;
extern int sox_new_effect;

struct _sox_cfg sox_cfg;
static struct _sox_cfg sox_cfg_old;

#define CONFIG_PROTO(prefix) \
static void prefix##_read_cfg(char *cfggrp, ConfigFile *cfg); \
static void prefix##_write_cfg(char *cfggrp, ConfigFile *cfg); \
static void prefix##_render_opts(GtkWidget *box); \
static void prefix##_ok(); \
static void prefix##_cancel(); \
static int prefix##_build_opts(char ***argv);

#define CONFIG_ENTRY(effect,prefix) \
	{ effect, prefix##_read_cfg, prefix##_write_cfg, prefix##_render_opts, prefix##_ok, prefix##_cancel, prefix##_build_opts }

CONFIG_PROTO(band);
CONFIG_PROTO(chorus);
CONFIG_PROTO(echo);
CONFIG_PROTO(echos);
CONFIG_PROTO(flanger);
CONFIG_PROTO(highp);
CONFIG_PROTO(lowp);
CONFIG_PROTO(phaser);
CONFIG_PROTO(reverb);
CONFIG_PROTO(vibro);

struct _configs sox_configs[] = {
	CONFIG_ENTRY("band",band),
	CONFIG_ENTRY("chorus",chorus),
	CONFIG_ENTRY("echo",echo),
	CONFIG_ENTRY("echos",echos),
	CONFIG_ENTRY("flanger",flanger),
	CONFIG_ENTRY("highp",highp),
	CONFIG_ENTRY("lowp",lowp),
	CONFIG_ENTRY("phaser",phaser),
	CONFIG_ENTRY("reverb",reverb),
	CONFIG_ENTRY("vibro",vibro),
	{ NULL, NULL, NULL }
};

/************* band effect ***************/
static struct _band {
	struct _band_cfg {
		gint noise;
		gfloat center;
		gint default_width;
		gfloat width;
	} cfg;

	GtkWidget *noise_tog;

	GtkWidget *center_box;
	GtkObject *center_adj;
	GtkWidget *center_label;
	GtkWidget *center_scale;
	GtkWidget *center_spin;

	GtkWidget *width_box;
	GtkObject *width_adj;
	GtkWidget *width_check;
	GtkWidget *width_scale;
	GtkWidget *width_spin;

} band;

static struct _band_cfg band_cfg_old;

static void band_read_cfg(char *cfggrp, ConfigFile *cfg) {
	band.cfg.noise = 0;
	band.cfg.center = 440;
	band.cfg.default_width = 1;
	band.cfg.width = 100;

	xmms_cfg_read_int(cfg, cfggrp, "band_noise", &band.cfg.noise);
	xmms_cfg_read_float(cfg, cfggrp, "band_center", &band.cfg.center);
	xmms_cfg_read_int(cfg, cfggrp, "band_default_width", &band.cfg.default_width);
	xmms_cfg_read_float(cfg, cfggrp, "band_width", &band.cfg.width);

	band_cfg_old = band.cfg;
}

static void band_write_cfg(char *cfggrp, ConfigFile *cfg) {
	xmms_cfg_write_int(cfg, cfggrp, "band_noise", band.cfg.noise);
	xmms_cfg_write_float(cfg, cfggrp, "band_center", band.cfg.center);
	xmms_cfg_write_int(cfg, cfggrp, "band_default_width", band.cfg.default_width);
	xmms_cfg_write_float(cfg, cfggrp, "band_width", band.cfg.width);

	band_cfg_old = band.cfg;
}

static void band_noise_tog_cb(GtkWidget *w, gpointer data) {
	band.cfg.noise = GTK_TOGGLE_BUTTON(band.noise_tog)->active;
	sox_restart_effect = 1;
}

static void band_center_adj_cb(GtkWidget *w, gpointer data) {
	band.cfg.center = GTK_ADJUSTMENT(band.center_adj)->value;
	sox_restart_effect = 1;
}

static void band_width_check_cb(GtkWidget *w, gpointer data) {
	band.cfg.default_width = !(GTK_TOGGLE_BUTTON(band.width_check)->active);
	sox_restart_effect = 1;
}

static void band_width_adj_cb(GtkWidget *w, gpointer data) {
	band.cfg.width = GTK_ADJUSTMENT(band.width_adj)->value;
	sox_restart_effect = 1;
}

static void band_ok() {
	band_cfg_old = band.cfg;
}

static void band_cancel() {
	band.cfg = band_cfg_old;
}

static void band_render_opts(GtkWidget *box) {
	/* Noise toggle */
	band.noise_tog = gtk_check_button_new_with_label("Noise");
	gtk_box_pack_start(GTK_BOX(box), band.noise_tog, FALSE, FALSE, 0);
	gtk_widget_show(band.noise_tog);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(band.noise_tog), band.cfg.noise);
	gtk_signal_connect(GTK_OBJECT(band.noise_tog), "toggled",
			(GtkSignalFunc)band_noise_tog_cb, NULL);

	/* Center entry */
	band.center_box = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), band.center_box, FALSE, FALSE, 0);
	gtk_widget_show(band.center_box);

	band.center_adj = gtk_adjustment_new(band.cfg.center, 0, 20000, 1, 5, 0);
	gtk_signal_connect(GTK_OBJECT(band.center_adj), "value_changed",
		GTK_SIGNAL_FUNC(band_center_adj_cb), NULL);

	band.center_label = gtk_label_new("Center Frequency:");
	gtk_box_pack_start(GTK_BOX(band.center_box), band.center_label, TRUE, TRUE, 0);
	gtk_widget_show(band.center_label);

	band.center_scale = gtk_hscale_new(GTK_ADJUSTMENT(band.center_adj));
	gtk_widget_set_usize(band.center_scale, 400, 30);
	gtk_scale_set_draw_value(GTK_SCALE(band.center_scale), FALSE);
	gtk_box_pack_start(GTK_BOX(band.center_box), band.center_scale, TRUE, TRUE, 0);
	gtk_widget_show(band.center_scale);

	band.center_spin = gtk_spin_button_new(GTK_ADJUSTMENT(band.center_adj), 1.0, 2);
	gtk_widget_set_usize(band.center_spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(band.center_box), band.center_spin, TRUE, TRUE, 0);
	gtk_widget_show(band.center_spin);

	/* Width toggle and entry */
	band.width_box = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), band.width_box, FALSE, FALSE, 0);
	gtk_widget_show(band.width_box);

	band.width_adj = gtk_adjustment_new(band.cfg.width, 0, 10000, 1, 5, 0);
	gtk_signal_connect(GTK_OBJECT(band.width_adj), "value_changed",
		GTK_SIGNAL_FUNC(band_width_adj_cb), NULL);

	band.width_check = gtk_check_button_new_with_label("Width:");
	gtk_box_pack_start(GTK_BOX(band.width_box), band.width_check, TRUE, TRUE, 0);
	gtk_widget_show(band.width_check);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(band.width_check), !band.cfg.default_width);
	gtk_signal_connect(GTK_OBJECT(band.width_check), "toggled",
			(GtkSignalFunc)band_width_check_cb, NULL);

	band.width_scale = gtk_hscale_new(GTK_ADJUSTMENT(band.width_adj));
	gtk_widget_set_usize(band.width_scale, 400, 30);
	gtk_scale_set_draw_value(GTK_SCALE(band.width_scale), FALSE);
	gtk_box_pack_start(GTK_BOX(band.width_box), band.width_scale, TRUE, TRUE, 0);
	gtk_widget_show(band.width_scale);

	band.width_spin = gtk_spin_button_new(GTK_ADJUSTMENT(band.width_adj), 1.0, 2);
	gtk_widget_set_usize(band.width_spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(band.width_box), band.width_spin, TRUE, TRUE, 0);
	gtk_widget_show(band.width_spin);
}

static int band_build_opts(char ***argv) {
	static char *iargv[] = { NULL, NULL, NULL, NULL };
	gint i;

	*argv = iargv;

	for (i = 0; iargv[i]; i++) {
		g_free(iargv[i]);
		iargv[i] = NULL;
	}

	i = 0;

	if (band.cfg.noise) iargv[i++] = g_strdup("-n");
	iargv[i++] = g_strdup_printf("%f", band.cfg.center);
	if (!band.cfg.default_width)
		iargv[i++] = g_strdup_printf("%f", band.cfg.width);

	return i;
}

/************* chorus effect ***************/
#define MAX_CHORUS 7

static struct _chorus {
	struct _chorus_cfg {
		gint num_chorus;
		gfloat gain_in;
		gfloat gain_out;
		struct _chorus_cfg_chorus {
			gfloat delay;
			gfloat decay;
			gfloat speed;
			gfloat depth;
			gint triangle;
		} chorus[MAX_CHORUS];
	} cfg;

	GtkObject *gain_in_adj, *gain_out_adj, *num_chorus_adj;
	GtkWidget *chorus_win;
	struct _chorus_chorus {
		GtkObject *delay_adj, *decay_adj, *speed_adj, *depth_adj;
		GtkWidget *triangle_tog;
	} chorus[MAX_CHORUS];
} chorus;

static struct _chorus_cfg chorus_cfg_old;

static void chorus_read_cfg(char *cfggrp, ConfigFile *cfg) {
	gchar buffer[40];
	gint i;

	chorus.cfg.num_chorus = 1;
	chorus.cfg.gain_in = 0.7;
	chorus.cfg.gain_out = 0.9;
	chorus.cfg.chorus[0].delay = 55.0;
	chorus.cfg.chorus[0].decay = 0.4;
	chorus.cfg.chorus[0].speed = 0.25;
	chorus.cfg.chorus[0].depth = 2.0;
	chorus.cfg.chorus[0].triangle = 1;

	xmms_cfg_read_int(cfg, cfggrp, "chorus_num_chorus", &chorus.cfg.num_chorus);
	xmms_cfg_read_float(cfg, cfggrp, "chorus_gain_in", &chorus.cfg.gain_in);
	xmms_cfg_read_float(cfg, cfggrp, "chorus_gain_out", &chorus.cfg.gain_out);
	for (i = 0; i < chorus.cfg.num_chorus; i++) {
		sprintf(buffer, "chorus_%d_delay", i);
		xmms_cfg_read_float(cfg, cfggrp, buffer, &chorus.cfg.chorus[i].delay);
		sprintf(buffer, "chorus_%d_decay", i);
		xmms_cfg_read_float(cfg, cfggrp, buffer, &chorus.cfg.chorus[i].decay);
		sprintf(buffer, "chorus_%d_speed", i);
		xmms_cfg_read_float(cfg, cfggrp, buffer, &chorus.cfg.chorus[i].speed);
		sprintf(buffer, "chorus_%d_depth", i);
		xmms_cfg_read_float(cfg, cfggrp, buffer, &chorus.cfg.chorus[i].depth);
		sprintf(buffer, "chorus_%d_triangle", i);
		xmms_cfg_read_int(cfg, cfggrp, buffer, &chorus.cfg.chorus[i].triangle);
	}

	chorus_cfg_old = chorus.cfg;
}

static void chorus_write_cfg(char *cfggrp, ConfigFile *cfg) {
	gchar buffer[40];
	gint i;

	xmms_cfg_write_int(cfg, cfggrp, "chorus_num_chorus", chorus.cfg.num_chorus);
	xmms_cfg_write_float(cfg, cfggrp, "chorus_gain_in", chorus.cfg.gain_in);
	xmms_cfg_write_float(cfg, cfggrp, "chorus_gain_out", chorus.cfg.gain_out);
	for (i = 0; i < chorus.cfg.num_chorus; i++) {
		sprintf(buffer, "chorus_%d_delay", i);
		xmms_cfg_write_float(cfg, cfggrp, buffer, chorus.cfg.chorus[i].delay);
		sprintf(buffer, "chorus_%d_decay", i);
		xmms_cfg_write_float(cfg, cfggrp, buffer, chorus.cfg.chorus[i].decay);
		sprintf(buffer, "chorus_%d_speed", i);
		xmms_cfg_write_float(cfg, cfggrp, buffer, chorus.cfg.chorus[i].speed);
		sprintf(buffer, "chorus_%d_depth", i);
		xmms_cfg_write_float(cfg, cfggrp, buffer, chorus.cfg.chorus[i].depth);
		sprintf(buffer, "chorus_%d_triangle", i);
		xmms_cfg_write_int(cfg, cfggrp, buffer, chorus.cfg.chorus[i].triangle);
	}

	chorus_cfg_old = chorus.cfg;
}

static void chorus_ok() {
	chorus_cfg_old = chorus.cfg;
}

static void chorus_cancel() {
	chorus.cfg = chorus_cfg_old;
}

static void chorus_adj_cb(GtkWidget *w, gpointer data) {
	*(gfloat*)data = GTK_ADJUSTMENT(w)->value;
	sox_restart_effect = 1;
}

static void chorus_magic_cb(GtkWidget *w, gpointer data) {
	gtk_widget_destroy(w);
}

static void chorus_triangle_cb(GtkWidget *w, gint data) {
	chorus.cfg.chorus[data].triangle =
		GTK_TOGGLE_BUTTON(chorus.chorus[data].triangle_tog)->active;
	sox_restart_effect = 1;
}

static void chorus_render_chorus() {
	GtkWidget *hbox, *vbox, *label, *spin, *radio;
	gint i;

	if (!chorus.chorus_win) return;

	gtk_container_foreach(GTK_CONTAINER(chorus.chorus_win),
		GTK_SIGNAL_FUNC(chorus_magic_cb), NULL);

	vbox = gtk_vbox_new(FALSE, 5);
	for (i = 0; i < chorus.cfg.num_chorus; i++) {
		hbox = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
		gtk_widget_show(hbox);

		/* Delay */
		chorus.chorus[i].delay_adj = gtk_adjustment_new(chorus.cfg.chorus[i].delay, 20.0, 100.0, 1.0, 1.0, 0);
		gtk_signal_connect(GTK_OBJECT(chorus.chorus[i].delay_adj), "value_changed",
			GTK_SIGNAL_FUNC(chorus_adj_cb), &chorus.cfg.chorus[i].delay);
		label = gtk_label_new("Delay:");
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);
		spin = gtk_spin_button_new(GTK_ADJUSTMENT(chorus.chorus[i].delay_adj), 0.0, 2);
		gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
		gtk_widget_show(spin);

		/* Decay */
		chorus.chorus[i].decay_adj = gtk_adjustment_new(chorus.cfg.chorus[i].decay, 0.0, 1.0, 0.01, 1.0, 0);
		gtk_signal_connect(GTK_OBJECT(chorus.chorus[i].decay_adj), "value_changed",
			GTK_SIGNAL_FUNC(chorus_adj_cb), &chorus.cfg.chorus[i].decay);
		label = gtk_label_new("Decay:");
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);
		spin = gtk_spin_button_new(GTK_ADJUSTMENT(chorus.chorus[i].decay_adj), 0.0, 2);
		gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
		gtk_widget_show(spin);

		/* Speed */
		chorus.chorus[i].speed_adj = gtk_adjustment_new(chorus.cfg.chorus[i].speed, 0.1, 5.0, 0.05, 1.0, 0);
		gtk_signal_connect(GTK_OBJECT(chorus.chorus[i].speed_adj), "value_changed",
			GTK_SIGNAL_FUNC(chorus_adj_cb), &chorus.cfg.chorus[i].speed);
		label = gtk_label_new("Speed:");
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);
		spin = gtk_spin_button_new(GTK_ADJUSTMENT(chorus.chorus[i].speed_adj), 0.0, 2);
		gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
		gtk_widget_show(spin);

		/* Depth */
		chorus.chorus[i].depth_adj = gtk_adjustment_new(chorus.cfg.chorus[i].depth, 0.0, 1000.0, 1.0, 1.0, 0);
		gtk_signal_connect(GTK_OBJECT(chorus.chorus[i].depth_adj), "value_changed",
			GTK_SIGNAL_FUNC(chorus_adj_cb), &chorus.cfg.chorus[i].depth);
		label = gtk_label_new("Depth:");
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);
		spin = gtk_spin_button_new(GTK_ADJUSTMENT(chorus.chorus[i].depth_adj), 0.0, 1);
		gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
		gtk_widget_show(spin);

		/* Sine/Triangle selection */
		radio = gtk_radio_button_new_with_label(NULL, "Sine");
		gtk_box_pack_start(GTK_BOX(hbox), radio, TRUE, TRUE, 0);
		gtk_widget_show(radio);

		chorus.chorus[i].triangle_tog = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio), "Triangle");
		gtk_box_pack_start(GTK_BOX(hbox), chorus.chorus[i].triangle_tog, TRUE, TRUE, 0);
		gtk_widget_show(chorus.chorus[i].triangle_tog);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chorus.chorus[i].triangle_tog), chorus.cfg.chorus[i].triangle);

		gtk_signal_connect(GTK_OBJECT(radio), "toggled",
			GTK_SIGNAL_FUNC(chorus_triangle_cb), GINT_TO_POINTER(i));
		gtk_signal_connect(GTK_OBJECT(chorus.chorus[i].triangle_tog), "toggled",
			GTK_SIGNAL_FUNC(chorus_triangle_cb), GINT_TO_POINTER(i));
	}

	gtk_widget_show(vbox);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(chorus.chorus_win),
		vbox);
}

static void chorus_num_chorus_adj_cb(GtkWidget *w, gpointer data) {
	chorus.cfg.num_chorus = GTK_ADJUSTMENT(chorus.num_chorus_adj)->value;
	chorus_render_chorus();
	sox_restart_effect = 1;
}

static void chorus_render_opts(GtkWidget *box) {
	GtkWidget *hbox, *spin, *label;

	chorus.chorus_win = 0;

	/* Gain in, Gain out, and number of choruses */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, FALSE, 0);
	gtk_widget_show(hbox);

	chorus.gain_in_adj = gtk_adjustment_new(chorus.cfg.gain_in, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(chorus.gain_in_adj), "value_changed",
		GTK_SIGNAL_FUNC(chorus_adj_cb), &chorus.cfg.gain_in);
	label = gtk_label_new("Gain in:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(chorus.gain_in_adj), 0.0, 4);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	chorus.gain_out_adj = gtk_adjustment_new(chorus.cfg.gain_out, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(chorus.gain_out_adj), "value_changed",
		GTK_SIGNAL_FUNC(chorus_adj_cb), &chorus.cfg.gain_out);
	label = gtk_label_new("Gain out:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(chorus.gain_out_adj), 0.0, 4);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	chorus.num_chorus_adj = gtk_adjustment_new(chorus.cfg.num_chorus, 1, 7, 1, 1, 0);
	gtk_signal_connect(GTK_OBJECT(chorus.num_chorus_adj), "value_changed",
		GTK_SIGNAL_FUNC(chorus_num_chorus_adj_cb), NULL);
	label = gtk_label_new("# of choruses:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(chorus.num_chorus_adj), 0.0, 0);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	chorus.chorus_win = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(chorus.chorus_win), 5);
	gtk_widget_set_usize(chorus.chorus_win, 0, 150);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(chorus.chorus_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	chorus_render_chorus();

	gtk_box_pack_start(GTK_BOX(box), chorus.chorus_win, TRUE, TRUE, 0);
	gtk_widget_show(chorus.chorus_win);
}

static int chorus_build_opts(char ***argv) {
	static char **iargv = NULL;
	gint i, j;

	if (iargv) {
		for (i = 0; iargv[i]; i++) {
			g_free(iargv[i]);
		}
		g_free(iargv);
	}

	iargv = g_malloc(sizeof(*iargv)*(3+(chorus.cfg.num_chorus*5)));
	*argv = iargv;

	i = 0;
	iargv[i++] = g_strdup_printf("%f", chorus.cfg.gain_in);
	iargv[i++] = g_strdup_printf("%f", chorus.cfg.gain_out);

	for (j = 0; j < chorus.cfg.num_chorus; j++) {
		iargv[i++] = g_strdup_printf("%f", chorus.cfg.chorus[j].delay);
		iargv[i++] = g_strdup_printf("%f", chorus.cfg.chorus[j].decay);
		iargv[i++] = g_strdup_printf("%f", chorus.cfg.chorus[j].speed);
		iargv[i++] = g_strdup_printf("%f", chorus.cfg.chorus[j].depth);
		if (chorus.cfg.chorus[j].triangle)
			iargv[i++] = g_strdup("-t");
		else
			iargv[i++] = g_strdup("-s");
	}

	iargv[i] = NULL;

	return i;
}

/*********** echo effect *************/
#define MAX_ECHOS 7

static struct _echo {
	struct _echo_cfg {
		gint num_echo;
		gfloat gain_in;
		gfloat gain_out;
		struct _echo_cfg_echo {
			gfloat delay;
			gfloat decay;
		} echo[MAX_ECHOS];
	} cfg;

	GtkObject *gain_in_adj, *gain_out_adj, *num_echo_adj;
	GtkWidget *echo_win;
	struct _echo_echo {
		GtkObject *delay_adj, *decay_adj;
	} echo[MAX_ECHOS];
} echo;

static struct _echo_cfg echo_cfg_old;

static void echo_read_cfg(char *cfggrp, ConfigFile *cfg) {
	gchar buffer[40];
	gint i;

	echo.cfg.num_echo = 1;
	echo.cfg.gain_in = 0.8;
	echo.cfg.gain_out = 0.88;
	echo.cfg.echo[0].delay = 60.0;
	echo.cfg.echo[0].decay = 0.4;

	xmms_cfg_read_int(cfg, cfggrp, "echo_num_echo", &echo.cfg.num_echo);
	xmms_cfg_read_float(cfg, cfggrp, "echo_gain_in", &echo.cfg.gain_in);
	xmms_cfg_read_float(cfg, cfggrp, "echo_gain_out", &echo.cfg.gain_out);
	for (i = 0; i < echo.cfg.num_echo; i++) {
		sprintf(buffer, "echo_%d_delay", i);
		xmms_cfg_read_float(cfg, cfggrp, buffer, &echo.cfg.echo[i].delay);
		sprintf(buffer, "echo_%d_decay", i);
		xmms_cfg_read_float(cfg, cfggrp, buffer, &echo.cfg.echo[i].decay);
	}

	echo_cfg_old = echo.cfg;
}

static void echo_write_cfg(char *cfggrp, ConfigFile *cfg) {
	gchar buffer[40];
	gint i;

	xmms_cfg_write_int(cfg, cfggrp, "echo_num_echo", echo.cfg.num_echo);
	xmms_cfg_write_float(cfg, cfggrp, "echo_gain_in", echo.cfg.gain_in);
	xmms_cfg_write_float(cfg, cfggrp, "echo_gain_out", echo.cfg.gain_out);
	for (i = 0; i < echo.cfg.num_echo; i++) {
		sprintf(buffer, "echo_%d_delay", i);
		xmms_cfg_write_float(cfg, cfggrp, buffer, echo.cfg.echo[i].delay);
		sprintf(buffer, "echo_%d_decay", i);
		xmms_cfg_write_float(cfg, cfggrp, buffer, echo.cfg.echo[i].decay);
	}

	echo_cfg_old = echo.cfg;
}

static void echo_ok() {
	echo_cfg_old = echo.cfg;
}

static void echo_cancel() {
	echo.cfg = echo_cfg_old;
}

static void echo_adj_cb(GtkWidget *w, gpointer data) {
	*(gfloat*)data = GTK_ADJUSTMENT(w)->value;
	sox_restart_effect = 1;
}

static void echo_magic_cb(GtkWidget *w, gpointer data) {
	gtk_widget_destroy(w);
}

static void echo_render_echo() {
	GtkWidget *hbox, *vbox, *label, *spin, *radio;
	gint i;

	if (!echo.echo_win) return;

	gtk_container_foreach(GTK_CONTAINER(echo.echo_win),
		GTK_SIGNAL_FUNC(echo_magic_cb), NULL);

	vbox = gtk_vbox_new(FALSE, 5);
	for (i = 0; i < echo.cfg.num_echo; i++) {
		hbox = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
		gtk_widget_show(hbox);

		/* Delay */
		echo.echo[i].delay_adj = gtk_adjustment_new(echo.cfg.echo[i].delay, 0.0000001, 10000.0, 1.0, 1.0, 0);
		gtk_signal_connect(GTK_OBJECT(echo.echo[i].delay_adj), "value_changed",
			GTK_SIGNAL_FUNC(echo_adj_cb), &echo.cfg.echo[i].delay);
		label = gtk_label_new("Delay:");
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);
		spin = gtk_spin_button_new(GTK_ADJUSTMENT(echo.echo[i].delay_adj), 0.0, 2);
		gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
		gtk_widget_show(spin);

		/* Decay */
		echo.echo[i].decay_adj = gtk_adjustment_new(echo.cfg.echo[i].decay, 0.0, 1.0, 0.01, 1.0, 0);
		gtk_signal_connect(GTK_OBJECT(echo.echo[i].decay_adj), "value_changed",
			GTK_SIGNAL_FUNC(echo_adj_cb), &echo.cfg.echo[i].decay);
		label = gtk_label_new("Decay:");
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);
		spin = gtk_spin_button_new(GTK_ADJUSTMENT(echo.echo[i].decay_adj), 0.0, 2);
		gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
		gtk_widget_show(spin);
	}

	gtk_widget_show(vbox);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(echo.echo_win),
		vbox);
}

static void echo_num_echo_adj_cb(GtkWidget *w, gpointer data) {
	echo.cfg.num_echo = GTK_ADJUSTMENT(echo.num_echo_adj)->value;
	echo_render_echo();
	sox_restart_effect = 1;
}

static void echo_render_opts(GtkWidget *box) {
	GtkWidget *hbox, *spin, *label;

	echo.echo_win = 0;

	/* Gain in, Gain out, and number of echoes */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, FALSE, 0);
	gtk_widget_show(hbox);

	echo.gain_in_adj = gtk_adjustment_new(echo.cfg.gain_in, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(echo.gain_in_adj), "value_changed",
		GTK_SIGNAL_FUNC(echo_adj_cb), &echo.cfg.gain_in);
	label = gtk_label_new("Gain in:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(echo.gain_in_adj), 0.0, 4);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	echo.gain_out_adj = gtk_adjustment_new(echo.cfg.gain_out, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(echo.gain_out_adj), "value_changed",
		GTK_SIGNAL_FUNC(echo_adj_cb), &echo.cfg.gain_out);
	label = gtk_label_new("Gain out:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(echo.gain_out_adj), 0.0, 4);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	echo.num_echo_adj = gtk_adjustment_new(echo.cfg.num_echo, 1, 7, 1, 1, 0);
	gtk_signal_connect(GTK_OBJECT(echo.num_echo_adj), "value_changed",
		GTK_SIGNAL_FUNC(echo_num_echo_adj_cb), NULL);
	label = gtk_label_new("# of echos:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(echo.num_echo_adj), 0.0, 0);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	echo.echo_win = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(echo.echo_win), 5);
	gtk_widget_set_usize(echo.echo_win, 0, 150);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(echo.echo_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	echo_render_echo();

	gtk_box_pack_start(GTK_BOX(box), echo.echo_win, TRUE, TRUE, 0);
	gtk_widget_show(echo.echo_win);
}

static int echo_build_opts(char ***argv) {
	static char **iargv = NULL;
	gint i, j;

	if (iargv) {
		for (i = 0; iargv[i]; i++) {
			g_free(iargv[i]);
		}
		g_free(iargv);
	}

	iargv = g_malloc(sizeof(*iargv)*(3+(echo.cfg.num_echo*2)));
	*argv = iargv;

	i = 0;
	iargv[i++] = g_strdup_printf("%f", echo.cfg.gain_in);
	iargv[i++] = g_strdup_printf("%f", echo.cfg.gain_out);

	for (j = 0; j < echo.cfg.num_echo; j++) {
		iargv[i++] = g_strdup_printf("%f", echo.cfg.echo[j].delay);
		iargv[i++] = g_strdup_printf("%f", echo.cfg.echo[j].decay);
	}

	iargv[i] = NULL;

	return i;
}

/*********** echos effect *************/
static struct _echos {
	struct _echos_cfg {
		gint num_echos;
		gfloat gain_in;
		gfloat gain_out;
		struct _echos_cfg_echos {
			gfloat delay;
			gfloat decay;
		} echos[MAX_ECHOS];
	} cfg;

	GtkObject *gain_in_adj, *gain_out_adj, *num_echos_adj;
	GtkWidget *echos_win;
	struct _echos_echos {
		GtkObject *delay_adj, *decay_adj;
	} echos[MAX_ECHOS];
} echos;

static struct _echos_cfg echos_cfg_old;

static void echos_read_cfg(char *cfggrp, ConfigFile *cfg) {
	gchar buffer[40];
	gint i;

	echos.cfg.num_echos = 2;
	echos.cfg.gain_in = 0.8;
	echos.cfg.gain_out = 0.7;
	echos.cfg.echos[0].delay = 40.0;
	echos.cfg.echos[0].decay = 0.25;
	echos.cfg.echos[1].delay = 63.0;
	echos.cfg.echos[1].decay = 0.3;

	xmms_cfg_read_int(cfg, cfggrp, "echos_num_echos", &echos.cfg.num_echos);
	xmms_cfg_read_float(cfg, cfggrp, "echos_gain_in", &echos.cfg.gain_in);
	xmms_cfg_read_float(cfg, cfggrp, "echos_gain_out", &echos.cfg.gain_out);
	for (i = 0; i < echos.cfg.num_echos; i++) {
		sprintf(buffer, "echos_%d_delay", i);
		xmms_cfg_read_float(cfg, cfggrp, buffer, &echos.cfg.echos[i].delay);
		sprintf(buffer, "echos_%d_decay", i);
		xmms_cfg_read_float(cfg, cfggrp, buffer, &echos.cfg.echos[i].decay);
	}

	echos_cfg_old = echos.cfg;
}

static void echos_write_cfg(char *cfggrp, ConfigFile *cfg) {
	gchar buffer[40];
	gint i;

	xmms_cfg_write_int(cfg, cfggrp, "echos_num_echos", echos.cfg.num_echos);
	xmms_cfg_write_float(cfg, cfggrp, "echos_gain_in", echos.cfg.gain_in);
	xmms_cfg_write_float(cfg, cfggrp, "echos_gain_out", echos.cfg.gain_out);
	for (i = 0; i < echos.cfg.num_echos; i++) {
		sprintf(buffer, "echos_%d_delay", i);
		xmms_cfg_write_float(cfg, cfggrp, buffer, echos.cfg.echos[i].delay);
		sprintf(buffer, "echos_%d_decay", i);
		xmms_cfg_write_float(cfg, cfggrp, buffer, echos.cfg.echos[i].decay);
	}

	echos_cfg_old = echos.cfg;
}

static void echos_ok() {
	echos_cfg_old = echos.cfg;
}

static void echos_cancel() {
	echos.cfg = echos_cfg_old;
}

static void echos_adj_cb(GtkWidget *w, gpointer data) {
	*(gfloat*)data = GTK_ADJUSTMENT(w)->value;
	sox_restart_effect = 1;
}

static void echos_magic_cb(GtkWidget *w, gpointer data) {
	gtk_widget_destroy(w);
}

static void echos_render_echos() {
	GtkWidget *hbox, *vbox, *label, *spin, *radio;
	gint i;

	if (!echos.echos_win) return;

	gtk_container_foreach(GTK_CONTAINER(echos.echos_win),
		GTK_SIGNAL_FUNC(echos_magic_cb), NULL);

	vbox = gtk_vbox_new(FALSE, 5);
	for (i = 0; i < echos.cfg.num_echos; i++) {
		hbox = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
		gtk_widget_show(hbox);

		/* Delay */
		echos.echos[i].delay_adj = gtk_adjustment_new(echos.cfg.echos[i].delay, 0.0000001, 10000.0, 1.0, 1.0, 0);
		gtk_signal_connect(GTK_OBJECT(echos.echos[i].delay_adj), "value_changed",
			GTK_SIGNAL_FUNC(echos_adj_cb), &echos.cfg.echos[i].delay);
		label = gtk_label_new("Delay:");
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);
		spin = gtk_spin_button_new(GTK_ADJUSTMENT(echos.echos[i].delay_adj), 0.0, 2);
		gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
		gtk_widget_show(spin);

		/* Decay */
		echos.echos[i].decay_adj = gtk_adjustment_new(echos.cfg.echos[i].decay, 0.0, 1.0, 0.01, 1.0, 0);
		gtk_signal_connect(GTK_OBJECT(echos.echos[i].decay_adj), "value_changed",
			GTK_SIGNAL_FUNC(echos_adj_cb), &echos.cfg.echos[i].decay);
		label = gtk_label_new("Decay:");
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);
		spin = gtk_spin_button_new(GTK_ADJUSTMENT(echos.echos[i].decay_adj), 0.0, 2);
		gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
		gtk_widget_show(spin);
	}

	gtk_widget_show(vbox);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(echos.echos_win),
		vbox);
}

static void echos_num_echos_adj_cb(GtkWidget *w, gpointer data) {
	echos.cfg.num_echos = GTK_ADJUSTMENT(echos.num_echos_adj)->value;
	echos_render_echos();
	sox_restart_effect = 1;
}

static void echos_render_opts(GtkWidget *box) {
	GtkWidget *hbox, *spin, *label;

	echos.echos_win = 0;

	/* Gain in, Gain out, and number of echoses */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, FALSE, 0);
	gtk_widget_show(hbox);

	echos.gain_in_adj = gtk_adjustment_new(echos.cfg.gain_in, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(echos.gain_in_adj), "value_changed",
		GTK_SIGNAL_FUNC(echos_adj_cb), &echos.cfg.gain_in);
	label = gtk_label_new("Gain in:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(echos.gain_in_adj), 0.0, 4);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	echos.gain_out_adj = gtk_adjustment_new(echos.cfg.gain_out, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(echos.gain_out_adj), "value_changed",
		GTK_SIGNAL_FUNC(echos_adj_cb), &echos.cfg.gain_out);
	label = gtk_label_new("Gain out:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(echos.gain_out_adj), 0.0, 4);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	echos.num_echos_adj = gtk_adjustment_new(echos.cfg.num_echos, 1, 7, 1, 1, 0);
	gtk_signal_connect(GTK_OBJECT(echos.num_echos_adj), "value_changed",
		GTK_SIGNAL_FUNC(echos_num_echos_adj_cb), NULL);
	label = gtk_label_new("# of echos:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(echos.num_echos_adj), 0.0, 0);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	echos.echos_win = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(echos.echos_win), 5);
	gtk_widget_set_usize(echos.echos_win, 0, 150);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(echos.echos_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	echos_render_echos();

	gtk_box_pack_start(GTK_BOX(box), echos.echos_win, TRUE, TRUE, 0);
	gtk_widget_show(echos.echos_win);
}

static int echos_build_opts(char ***argv) {
	static char **iargv = NULL;
	gint i, j;

	if (iargv) {
		for (i = 0; iargv[i]; i++) {
			g_free(iargv[i]);
		}
		g_free(iargv);
	}

	iargv = g_malloc(sizeof(*iargv)*(3+(echos.cfg.num_echos*2)));
	*argv = iargv;

	i = 0;
	iargv[i++] = g_strdup_printf("%f", echos.cfg.gain_in);
	iargv[i++] = g_strdup_printf("%f", echos.cfg.gain_out);

	for (j = 0; j < echos.cfg.num_echos; j++) {
		iargv[i++] = g_strdup_printf("%f", echos.cfg.echos[j].delay);
		iargv[i++] = g_strdup_printf("%f", echos.cfg.echos[j].decay);
	}

	iargv[i] = NULL;

	return i;
}

/************ flanger effect ************/
static struct _flanger {
	struct _flanger_cfg {
		gfloat gain_in, gain_out;
		gfloat delay, decay, speed;
		gint triangle;
	} cfg;

	GtkObject *gain_in_adj, *gain_out_adj;
	GtkObject *delay_adj, *decay_adj, *speed_adj;
	GtkWidget *triangle_tog;
} flanger;

static struct _flanger_cfg flanger_cfg_old;

static void flanger_read_cfg(char *cfggrp, ConfigFile *cfg) {
	flanger.cfg.gain_in = 0.8;
	flanger.cfg.gain_out = 0.88;
	flanger.cfg.delay = 3.0;
	flanger.cfg.decay = 0.4;
	flanger.cfg.speed = 0.5;
	flanger.cfg.triangle = 1;

	xmms_cfg_read_float(cfg, cfggrp, "flanger_gain_in", &flanger.cfg.gain_in);
	xmms_cfg_read_float(cfg, cfggrp, "flanger_gain_out", &flanger.cfg.gain_out);
	xmms_cfg_read_float(cfg, cfggrp, "flanger_delay", &flanger.cfg.delay);
	xmms_cfg_read_float(cfg, cfggrp, "flanger_decay", &flanger.cfg.decay);
	xmms_cfg_read_float(cfg, cfggrp, "flanger_speed", &flanger.cfg.speed);
	xmms_cfg_read_int(cfg, cfggrp, "flanger_triangle", &flanger.cfg.triangle);

	flanger_cfg_old = flanger.cfg;
}

static void flanger_write_cfg(char *cfggrp, ConfigFile *cfg) {
	xmms_cfg_write_float(cfg, cfggrp, "flanger_gain_in", flanger.cfg.gain_in);
	xmms_cfg_write_float(cfg, cfggrp, "flanger_gain_out", flanger.cfg.gain_out);
	xmms_cfg_write_float(cfg, cfggrp, "flanger_delay", flanger.cfg.delay);
	xmms_cfg_write_float(cfg, cfggrp, "flanger_decay", flanger.cfg.decay);
	xmms_cfg_write_float(cfg, cfggrp, "flanger_speed", flanger.cfg.speed);
	xmms_cfg_write_int(cfg, cfggrp, "flanger_triangle", flanger.cfg.triangle);

	flanger_cfg_old = flanger.cfg;
}

static void flanger_triangle_tog_cb(GtkWidget *w, gpointer data) {
	flanger.cfg.triangle = GTK_TOGGLE_BUTTON(flanger.triangle_tog)->active;
	sox_restart_effect = 1;
}

static void flanger_adj_cb(GtkWidget *w, gpointer data) {
	*(gfloat*)data = GTK_ADJUSTMENT(w)->value;
	sox_restart_effect = 1;
}

static void flanger_ok() {
	flanger_cfg_old = flanger.cfg;
}

static void flanger_cancel() {
	flanger.cfg = flanger_cfg_old;
}

static void flanger_render_opts(GtkWidget *box) {
	GtkWidget *hbox, *label, *spin, *radio;

	/* Gain in/gain out */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	flanger.gain_in_adj = gtk_adjustment_new(flanger.cfg.gain_in, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(flanger.gain_in_adj), "value-changed",
		GTK_SIGNAL_FUNC(flanger_adj_cb), &flanger.cfg.gain_in);
	label = gtk_label_new("Gain in:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(flanger.gain_in_adj), 0.0, 4);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	flanger.gain_out_adj = gtk_adjustment_new(flanger.cfg.gain_out, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(flanger.gain_out_adj), "value-changed",
		GTK_SIGNAL_FUNC(flanger_adj_cb), &flanger.cfg.gain_out);
	label = gtk_label_new("Gain out:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(flanger.gain_out_adj), 0.0, 4);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);


	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);
	/* Delay */
	flanger.delay_adj = gtk_adjustment_new(flanger.cfg.delay, 0.0, 5.0, 0.1, 0.1, 0.0);
	gtk_signal_connect(GTK_OBJECT(flanger.delay_adj), "value-changed",
		GTK_SIGNAL_FUNC(flanger_adj_cb), &flanger.cfg.delay);
	label = gtk_label_new("Delay:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(flanger.delay_adj), 0.0, 2);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	/* Decay */
	flanger.decay_adj = gtk_adjustment_new(flanger.cfg.decay, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(flanger.decay_adj), "value-changed",
		GTK_SIGNAL_FUNC(flanger_adj_cb), &flanger.cfg.decay);
	label = gtk_label_new("Decay:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(flanger.decay_adj), 0.0, 2);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	/* Speed */
	flanger.speed_adj = gtk_adjustment_new(flanger.cfg.speed, 0.1, 2.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(flanger.speed_adj), "value-changed",
		GTK_SIGNAL_FUNC(flanger_adj_cb), &flanger.cfg.speed);
	label = gtk_label_new("Speed:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(flanger.speed_adj), 0.0, 4);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	/* Sine/Triangle */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	radio = gtk_radio_button_new_with_label(NULL, "Sine");
	gtk_box_pack_start(GTK_BOX(hbox), radio, TRUE, TRUE, 0);
	gtk_widget_show(radio);

	flanger.triangle_tog = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio), "Triangle");
	gtk_box_pack_start(GTK_BOX(hbox), flanger.triangle_tog, TRUE, TRUE, 0);
	gtk_widget_show(flanger.triangle_tog);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(flanger.triangle_tog), flanger.cfg.triangle);

	gtk_signal_connect(GTK_OBJECT(radio), "toggled",
		GTK_SIGNAL_FUNC(flanger_triangle_tog_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(flanger.triangle_tog), "toggled",
		GTK_SIGNAL_FUNC(flanger_triangle_tog_cb), NULL);
}

static int flanger_build_opts(char ***argv) {
	static char *iargv[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
	gint i;

	*argv = iargv;

	for (i = 0; iargv[i]; i++) {
		g_free(iargv[i]);
		iargv[i] = NULL;
	}

	i = 0;

	iargv[i++] = g_strdup_printf("%f", flanger.cfg.gain_in);
	iargv[i++] = g_strdup_printf("%f", flanger.cfg.gain_out);
	iargv[i++] = g_strdup_printf("%f", flanger.cfg.delay);
	iargv[i++] = g_strdup_printf("%f", flanger.cfg.decay);
	iargv[i++] = g_strdup_printf("%f", flanger.cfg.speed);
	if (flanger.cfg.triangle)
		iargv[i++] = g_strdup("-t");
	else
		iargv[i++] = g_strdup("-s");

	return i;
}

/************* highp effect (high pass) ***************/
static struct _highp {
	struct _highp_cfg {
		gfloat center;
	} cfg;

	GtkObject *center_adj;
} highp;

static struct _highp_cfg highp_cfg_old;

static void highp_read_cfg(char *cfggrp, ConfigFile *cfg) {
	highp.cfg.center = 11025;

	xmms_cfg_read_float(cfg, cfggrp, "highp_center", &highp.cfg.center);

	highp_cfg_old = highp.cfg;
}

static void highp_write_cfg(char *cfggrp, ConfigFile *cfg) {
	xmms_cfg_write_float(cfg, cfggrp, "highp_center", highp.cfg.center);

	highp_cfg_old = highp.cfg;
}

static void highp_center_adj_cb(GtkWidget *w, gpointer data) {
	highp.cfg.center = GTK_ADJUSTMENT(highp.center_adj)->value;
	sox_restart_effect = 1;
}

static void highp_ok() {
	highp_cfg_old = highp.cfg;
}

static void highp_cancel() {
	highp.cfg = highp_cfg_old;
}

static void highp_render_opts(GtkWidget *box) {
	GtkWidget *hbox, *label, *hscale, *spin;

	/* Center entry */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	highp.center_adj = gtk_adjustment_new(highp.cfg.center, 0, 44100*2, 100, 50, 0);
	gtk_signal_connect(GTK_OBJECT(highp.center_adj), "value_changed",
		GTK_SIGNAL_FUNC(highp_center_adj_cb), NULL);

	label = gtk_label_new("Center:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	hscale = gtk_hscale_new(GTK_ADJUSTMENT(highp.center_adj));
	gtk_widget_set_usize(hscale, 400, 30);
	gtk_scale_set_draw_value(GTK_SCALE(hscale), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
	gtk_widget_show(hscale);

	spin = gtk_spin_button_new(GTK_ADJUSTMENT(highp.center_adj), 1.0, 2);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);
}

static int highp_build_opts(char ***argv) {
	static char *iargv[] = { NULL, NULL };
	gint i;

	*argv = iargv;

	for (i = 0; iargv[i]; i++) {
		g_free(iargv[i]);
		iargv[i] = NULL;
	}

	i = 0;

	iargv[i++] = g_strdup_printf("%f", highp.cfg.center);

	return i;
}

/************* lowp effect (low pass) ***************/
static struct _lowp {
	struct _lowp_cfg {
		gfloat center;
	} cfg;

	GtkObject *center_adj;
} lowp;

static struct _lowp_cfg lowp_cfg_old;

static void lowp_read_cfg(char *cfggrp, ConfigFile *cfg) {
	lowp.cfg.center = 11025;

	xmms_cfg_read_float(cfg, cfggrp, "lowp_center", &lowp.cfg.center);

	lowp_cfg_old = lowp.cfg;
}

static void lowp_write_cfg(char *cfggrp, ConfigFile *cfg) {
	xmms_cfg_write_float(cfg, cfggrp, "lowp_center", lowp.cfg.center);

	lowp_cfg_old = lowp.cfg;
}

static void lowp_center_adj_cb(GtkWidget *w, gpointer data) {
	lowp.cfg.center = GTK_ADJUSTMENT(lowp.center_adj)->value;
	sox_restart_effect = 1;
}

static void lowp_ok() {
	lowp_cfg_old = lowp.cfg;
}

static void lowp_cancel() {
	lowp.cfg = lowp_cfg_old;
}

static void lowp_render_opts(GtkWidget *box) {
	GtkWidget *hbox, *label, *hscale, *spin;

	/* Center entry */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	lowp.center_adj = gtk_adjustment_new(lowp.cfg.center, 0, 44100*2, 100, 50, 0);
	gtk_signal_connect(GTK_OBJECT(lowp.center_adj), "value_changed",
		GTK_SIGNAL_FUNC(lowp_center_adj_cb), NULL);

	label = gtk_label_new("Center:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	hscale = gtk_hscale_new(GTK_ADJUSTMENT(lowp.center_adj));
	gtk_widget_set_usize(hscale, 400, 30);
	gtk_scale_set_draw_value(GTK_SCALE(hscale), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
	gtk_widget_show(hscale);

	spin = gtk_spin_button_new(GTK_ADJUSTMENT(lowp.center_adj), 1.0, 2);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);
}

static int lowp_build_opts(char ***argv) {
	static char *iargv[] = { NULL, NULL };
	gint i;

	*argv = iargv;

	for (i = 0; iargv[i]; i++) {
		g_free(iargv[i]);
		iargv[i] = NULL;
	}

	i = 0;

	iargv[i++] = g_strdup_printf("%f", lowp.cfg.center);

	return i;
}


/************ phaser effect ************/
static struct _phaser {
	struct _phaser_cfg {
		gfloat gain_in, gain_out;
		gfloat delay, decay, speed;
		gint triangle;
	} cfg;

	GtkObject *gain_in_adj, *gain_out_adj;
	GtkObject *delay_adj, *decay_adj, *speed_adj;
	GtkWidget *triangle_tog;
} phaser;

static struct _phaser_cfg phaser_cfg_old;

static void phaser_read_cfg(char *cfggrp, ConfigFile *cfg) {
	phaser.cfg.gain_in = 0.89;
	phaser.cfg.gain_out = 0.85;
	phaser.cfg.delay = 1.0;
	phaser.cfg.decay = 0.24;
	phaser.cfg.speed = 2.0;
	phaser.cfg.triangle = 1;

	xmms_cfg_read_float(cfg, cfggrp, "phaser_gain_in", &phaser.cfg.gain_in);
	xmms_cfg_read_float(cfg, cfggrp, "phaser_gain_out", &phaser.cfg.gain_out);
	xmms_cfg_read_float(cfg, cfggrp, "phaser_delay", &phaser.cfg.delay);
	xmms_cfg_read_float(cfg, cfggrp, "phaser_decay", &phaser.cfg.decay);
	xmms_cfg_read_float(cfg, cfggrp, "phaser_speed", &phaser.cfg.speed);
	xmms_cfg_read_int(cfg, cfggrp, "phaser_triangle", &phaser.cfg.triangle);

	phaser_cfg_old = phaser.cfg;
}

static void phaser_write_cfg(char *cfggrp, ConfigFile *cfg) {
	xmms_cfg_write_float(cfg, cfggrp, "phaser_gain_in", phaser.cfg.gain_in);
	xmms_cfg_write_float(cfg, cfggrp, "phaser_gain_out", phaser.cfg.gain_out);
	xmms_cfg_write_float(cfg, cfggrp, "phaser_delay", phaser.cfg.delay);
	xmms_cfg_write_float(cfg, cfggrp, "phaser_decay", phaser.cfg.decay);
	xmms_cfg_write_float(cfg, cfggrp, "phaser_speed", phaser.cfg.speed);
	xmms_cfg_write_int(cfg, cfggrp, "phaser_triangle", phaser.cfg.triangle);

	phaser_cfg_old = phaser.cfg;
}

static void phaser_triangle_tog_cb(GtkWidget *w, gpointer data) {
	phaser.cfg.triangle = GTK_TOGGLE_BUTTON(phaser.triangle_tog)->active;
	sox_restart_effect = 1;
}

static void phaser_adj_cb(GtkWidget *w, gpointer data) {
	*(gfloat*)data = GTK_ADJUSTMENT(w)->value;
	sox_restart_effect = 1;
}

static void phaser_ok() {
	phaser_cfg_old = phaser.cfg;
}

static void phaser_cancel() {
	phaser.cfg = phaser_cfg_old;
}

static void phaser_render_opts(GtkWidget *box) {
	GtkWidget *hbox, *label, *spin, *radio;

	/* Gain in/gain out */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	phaser.gain_in_adj = gtk_adjustment_new(phaser.cfg.gain_in, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(phaser.gain_in_adj), "value-changed",
		GTK_SIGNAL_FUNC(phaser_adj_cb), &phaser.cfg.gain_in);
	label = gtk_label_new("Gain in:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(phaser.gain_in_adj), 0.0, 4);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	phaser.gain_out_adj = gtk_adjustment_new(phaser.cfg.gain_out, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(phaser.gain_out_adj), "value-changed",
		GTK_SIGNAL_FUNC(phaser_adj_cb), &phaser.cfg.gain_out);
	label = gtk_label_new("Gain out:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(phaser.gain_out_adj), 0.0, 4);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);


	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);
	/* Delay */
	phaser.delay_adj = gtk_adjustment_new(phaser.cfg.delay, 0.0, 5.0, 0.1, 0.1, 0.0);
	gtk_signal_connect(GTK_OBJECT(phaser.delay_adj), "value-changed",
		GTK_SIGNAL_FUNC(phaser_adj_cb), &phaser.cfg.delay);
	label = gtk_label_new("Delay:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(phaser.delay_adj), 0.0, 2);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	/* Decay */
	phaser.decay_adj = gtk_adjustment_new(phaser.cfg.decay, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(phaser.decay_adj), "value-changed",
		GTK_SIGNAL_FUNC(phaser_adj_cb), &phaser.cfg.decay);
	label = gtk_label_new("Decay:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(phaser.decay_adj), 0.0, 2);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	/* Speed */
	phaser.speed_adj = gtk_adjustment_new(phaser.cfg.speed, 0.1, 2.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(phaser.speed_adj), "value-changed",
		GTK_SIGNAL_FUNC(phaser_adj_cb), &phaser.cfg.speed);
	label = gtk_label_new("Speed:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(phaser.speed_adj), 0.0, 4);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	/* Sine/Triangle */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	radio = gtk_radio_button_new_with_label(NULL, "Sine");
	gtk_box_pack_start(GTK_BOX(hbox), radio, TRUE, TRUE, 0);
	gtk_widget_show(radio);

	phaser.triangle_tog = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio), "Triangle");
	gtk_box_pack_start(GTK_BOX(hbox), phaser.triangle_tog, TRUE, TRUE, 0);
	gtk_widget_show(phaser.triangle_tog);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(phaser.triangle_tog), phaser.cfg.triangle);

	gtk_signal_connect(GTK_OBJECT(radio), "toggled",
		GTK_SIGNAL_FUNC(phaser_triangle_tog_cb), NULL);
	gtk_signal_connect(GTK_OBJECT(phaser.triangle_tog), "toggled",
		GTK_SIGNAL_FUNC(phaser_triangle_tog_cb), NULL);
}

static int phaser_build_opts(char ***argv) {
	static char *iargv[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
	gint i;

	*argv = iargv;

	for (i = 0; iargv[i]; i++) {
		g_free(iargv[i]);
		iargv[i] = NULL;
	}

	i = 0;

	iargv[i++] = g_strdup_printf("%f", phaser.cfg.gain_in);
	iargv[i++] = g_strdup_printf("%f", phaser.cfg.gain_out);
	iargv[i++] = g_strdup_printf("%f", phaser.cfg.delay);
	iargv[i++] = g_strdup_printf("%f", phaser.cfg.decay);
	iargv[i++] = g_strdup_printf("%f", phaser.cfg.speed);
	if (phaser.cfg.triangle)
		iargv[i++] = g_strdup("-t");
	else
		iargv[i++] = g_strdup("-s");

	return i;
}

/*********** reverb effect *************/
#define MAXREVERBS 8

static struct _reverb {
	struct _reverb_cfg {
		gint num_reverb;
		gfloat gain_out;
		gfloat reverb_time;
		struct _reverb_cfg_reverb {
			gfloat delay;
		} reverb[MAXREVERBS];
	} cfg;

	GtkObject *gain_out_adj, *reverb_time_adj, *num_reverb_adj;
	GtkWidget *reverb_win;
	struct _reverb_reverb {
		GtkObject *delay_adj;
	} reverb[MAXREVERBS];
} reverb;

static struct _reverb_cfg reverb_cfg_old;

static void reverb_read_cfg(char *cfggrp, ConfigFile *cfg) {
	gchar buffer[40];
	gint i;

	reverb.cfg.num_reverb = 4;
	reverb.cfg.gain_out = 1.0;
	reverb.cfg.reverb_time = 600.0;
	reverb.cfg.reverb[0].delay = 180.0;
	reverb.cfg.reverb[1].delay = 200.0;
	reverb.cfg.reverb[2].delay = 220.0;
	reverb.cfg.reverb[3].delay = 240.0;

	xmms_cfg_read_int(cfg, cfggrp, "reverb_num_reverb", &reverb.cfg.num_reverb);
	xmms_cfg_read_float(cfg, cfggrp, "reverb_gain_out", &reverb.cfg.gain_out);
	xmms_cfg_read_float(cfg, cfggrp, "reverb_time", &reverb.cfg.reverb_time);
	for (i = 0; i < reverb.cfg.num_reverb; i++) {
		sprintf(buffer, "reverb_%d_delay", i);
		xmms_cfg_read_float(cfg, cfggrp, buffer, &reverb.cfg.reverb[i].delay);
	}

	reverb_cfg_old = reverb.cfg;
}

static void reverb_write_cfg(char *cfggrp, ConfigFile *cfg) {
	gchar buffer[40];
	gint i;

	xmms_cfg_write_int(cfg, cfggrp, "reverb_num_reverb", reverb.cfg.num_reverb);
	xmms_cfg_write_float(cfg, cfggrp, "reverb_gain_out", reverb.cfg.gain_out);
	xmms_cfg_write_float(cfg, cfggrp, "reverb_time", reverb.cfg.reverb_time);
	for (i = 0; i < reverb.cfg.num_reverb; i++) {
		sprintf(buffer, "reverb_%d_delay", i);
		xmms_cfg_write_float(cfg, cfggrp, buffer, reverb.cfg.reverb[i].delay);
	}

	reverb_cfg_old = reverb.cfg;
}

static void reverb_ok() {
	reverb_cfg_old = reverb.cfg;
}

static void reverb_cancel() {
	reverb.cfg = reverb_cfg_old;
}

static void reverb_adj_cb(GtkWidget *w, gpointer data) {
	*(gfloat*)data = GTK_ADJUSTMENT(w)->value;
	sox_restart_effect = 1;
}

static void reverb_magic_cb(GtkWidget *w, gpointer data) {
	gtk_widget_destroy(w);
}

static void reverb_render_reverb() {
	GtkWidget *hbox, *vbox, *label, *spin, *radio;
	gint i;

	if (!reverb.reverb_win) return;

	gtk_container_foreach(GTK_CONTAINER(reverb.reverb_win),
		GTK_SIGNAL_FUNC(reverb_magic_cb), NULL);

	vbox = gtk_vbox_new(FALSE, 5);
	for (i = 0; i < reverb.cfg.num_reverb; i++) {
		hbox = gtk_hbox_new(FALSE, 5);
		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
		gtk_widget_show(hbox);

		/* Delay */
		reverb.reverb[i].delay_adj = gtk_adjustment_new(reverb.cfg.reverb[i].delay, 0.0000001, 10000.0, 1.0, 1.0, 0);
		gtk_signal_connect(GTK_OBJECT(reverb.reverb[i].delay_adj), "value_changed",
			GTK_SIGNAL_FUNC(reverb_adj_cb), &reverb.cfg.reverb[i].delay);
		label = gtk_label_new("Delay:");
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);
		spin = gtk_spin_button_new(GTK_ADJUSTMENT(reverb.reverb[i].delay_adj), 0.0, 2);
		gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
		gtk_widget_show(spin);
	}

	gtk_widget_show(vbox);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(reverb.reverb_win),
		vbox);
}

static void reverb_num_reverb_adj_cb(GtkWidget *w, gpointer data) {
	reverb.cfg.num_reverb = GTK_ADJUSTMENT(reverb.num_reverb_adj)->value;
	reverb_render_reverb();
	sox_restart_effect = 1;
}

static void reverb_render_opts(GtkWidget *box) {
	GtkWidget *hbox, *spin, *label;

	reverb.reverb_win = 0;

	/* Gain out, reverb time, and number of reverbes */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, FALSE, 0);
	gtk_widget_show(hbox);

	reverb.gain_out_adj = gtk_adjustment_new(reverb.cfg.gain_out, 0.0, 1.0, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(reverb.gain_out_adj), "value_changed",
		GTK_SIGNAL_FUNC(reverb_adj_cb), &reverb.cfg.gain_out);
	label = gtk_label_new("Gain out:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(reverb.gain_out_adj), 0.0, 4);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	reverb.reverb_time_adj = gtk_adjustment_new(reverb.cfg.reverb_time, 0.0, 100000.0, 10, 10, 0.0);
	gtk_signal_connect(GTK_OBJECT(reverb.reverb_time_adj), "value_changed",
		GTK_SIGNAL_FUNC(reverb_adj_cb), &reverb.cfg.reverb_time);
	label = gtk_label_new("Reverb time:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(reverb.reverb_time_adj), 0.0, 1);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	reverb.num_reverb_adj = gtk_adjustment_new(reverb.cfg.num_reverb, 1, 7, 1, 1, 0);
	gtk_signal_connect(GTK_OBJECT(reverb.num_reverb_adj), "value_changed",
		GTK_SIGNAL_FUNC(reverb_num_reverb_adj_cb), NULL);
	label = gtk_label_new("# of reverbs:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);
	spin = gtk_spin_button_new(GTK_ADJUSTMENT(reverb.num_reverb_adj), 0.0, 0);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	reverb.reverb_win = gtk_scrolled_window_new(NULL, NULL);
	gtk_container_set_border_width(GTK_CONTAINER(reverb.reverb_win), 5);
	gtk_widget_set_usize(reverb.reverb_win, 0, 150);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(reverb.reverb_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	reverb_render_reverb();

	gtk_box_pack_start(GTK_BOX(box), reverb.reverb_win, TRUE, TRUE, 0);
	gtk_widget_show(reverb.reverb_win);
}

static int reverb_build_opts(char ***argv) {
	static char **iargv = NULL;
	gint i, j;

	if (iargv) {
		for (i = 0; iargv[i]; i++) {
			g_free(iargv[i]);
		}
		g_free(iargv);
	}

	iargv = g_malloc(sizeof(*iargv)*(3+reverb.cfg.num_reverb));
	*argv = iargv;

	i = 0;
	iargv[i++] = g_strdup_printf("%f", reverb.cfg.gain_out);
	iargv[i++] = g_strdup_printf("%f", reverb.cfg.reverb_time);

	for (j = 0; j < reverb.cfg.num_reverb; j++) {
		iargv[i++] = g_strdup_printf("%f", reverb.cfg.reverb[j].delay);
	}

	iargv[i] = NULL;

	return i;
}

/************* vibro effect ***************/
static struct _vibro {
	struct _vibro_cfg {
		gfloat speed;
		gfloat depth;
	} cfg;

	GtkObject *speed_adj, *depth_adj;
} vibro;

static struct _vibro_cfg vibro_cfg_old;

static void vibro_read_cfg(char *cfggrp, ConfigFile *cfg) {
	vibro.cfg.speed = 5;
	vibro.cfg.depth = 0.5;

	xmms_cfg_read_float(cfg, cfggrp, "vibro_speed", &vibro.cfg.speed);
	xmms_cfg_read_float(cfg, cfggrp, "vibro_depth", &vibro.cfg.depth);

	vibro_cfg_old = vibro.cfg;
}

static void vibro_write_cfg(char *cfggrp, ConfigFile *cfg) {
	xmms_cfg_write_float(cfg, cfggrp, "vibro_speed", vibro.cfg.speed);
	xmms_cfg_write_float(cfg, cfggrp, "vibro_depth", vibro.cfg.depth);

	vibro_cfg_old = vibro.cfg;
}

static void vibro_adj_cb(GtkWidget *w, gpointer data) {
	*(float*)data = GTK_ADJUSTMENT(w)->value;
	sox_restart_effect = 1;
}

static void vibro_ok() {
	vibro_cfg_old = vibro.cfg;
}

static void vibro_cancel() {
	vibro.cfg = vibro_cfg_old;
}

static void vibro_render_opts(GtkWidget *box) {
	GtkWidget *hbox, *label, *hscale, *spin;

	/* speed entry */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	vibro.speed_adj = gtk_adjustment_new(vibro.cfg.speed, 0.0011, 30.0, 1, 1, 0);
	gtk_signal_connect(GTK_OBJECT(vibro.speed_adj), "value_changed",
		GTK_SIGNAL_FUNC(vibro_adj_cb), &vibro.cfg.speed);

	label = gtk_label_new("Speed:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	hscale = gtk_hscale_new(GTK_ADJUSTMENT(vibro.speed_adj));
	gtk_widget_set_usize(hscale, 400, 30);
	gtk_scale_set_draw_value(GTK_SCALE(hscale), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
	gtk_widget_show(hscale);

	spin = gtk_spin_button_new(GTK_ADJUSTMENT(vibro.speed_adj), 1.0, 2);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);

	/* depth entry */
	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	vibro.depth_adj = gtk_adjustment_new(vibro.cfg.depth, 0.0, 1.0, 0.01, 0.01, 0);
	gtk_signal_connect(GTK_OBJECT(vibro.depth_adj), "value_changed",
		GTK_SIGNAL_FUNC(vibro_adj_cb), &vibro.cfg.depth);

	label = gtk_label_new("Depth:");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	hscale = gtk_hscale_new(GTK_ADJUSTMENT(vibro.depth_adj));
	gtk_widget_set_usize(hscale, 400, 30);
	gtk_scale_set_draw_value(GTK_SCALE(hscale), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
	gtk_widget_show(hscale);

	spin = gtk_spin_button_new(GTK_ADJUSTMENT(vibro.depth_adj), 1.0, 2);
	gtk_widget_set_usize(spin, 75, 0);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);
	gtk_widget_show(spin);
}

static int vibro_build_opts(char ***argv) {
	static char *iargv[] = { NULL, NULL, NULL };
	gint i;

	*argv = iargv;

	for (i = 0; iargv[i]; i++) {
		g_free(iargv[i]);
		iargv[i] = NULL;
	}

	i = 0;

	iargv[i++] = g_strdup_printf("%f", vibro.cfg.speed);
	iargv[i++] = g_strdup_printf("%f", vibro.cfg.depth);

	return i;
}

/************ Main Config **************/
static GtkWidget *configure_win;
static GtkWidget *effect;
static GtkWidget *win_box;
static GtkWidget *button_box;
static GtkWidget *ok_button;
static GtkWidget *cancel_button;

static int config_read = 0;

void sox_read_config() {
	ConfigFile *cfg;
	gchar *filename;
	gint i;

	if (!config_read) {
		sox_cfg.effect_number = 3;

		filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
		cfg = xmms_cfg_open_file(filename);
		if (cfg) {
			xmms_cfg_read_int(cfg, "sox-effect", "effect", &sox_cfg.effect_number);
			for (i = 0; sox_configs[i].name; i++) {
				sox_configs[i].read_cfg("sox-effect", cfg);
			}
			xmms_cfg_free(cfg);
		}
		g_free(filename);
		config_read = TRUE;
	}
}

void sox_write_config() {
	ConfigFile *cfg;
	gchar *filename;
	gint i;

	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	cfg = xmms_cfg_open_file(filename);
	if (!cfg) cfg = xmms_cfg_new();

	xmms_cfg_write_int(cfg, "sox-effect", "effect", sox_cfg.effect_number);
	for (i = 0; sox_configs[i].name; i++) {
		sox_configs[i].write_cfg("sox-effect", cfg);
	}
	xmms_cfg_write_file(cfg, filename);
	xmms_cfg_free(cfg);
	g_free(filename);
}

static void sox_effect_cb(GtkNotebook *notebook, GtkNotebookPage *page, gint page_num, gpointer user_data) {
	sox_cfg.effect_number = page_num;
	sox_restart_effect = 1;
}

static void ok_button_cb(GtkWidget *w, gpointer data) {
	gint i;

	gtk_signal_disconnect_by_func(GTK_OBJECT(effect),
		GTK_SIGNAL_FUNC(sox_effect_cb), NULL);
	for (i = 0; sox_configs[i].name; i++) {
		sox_configs[i].ok();
	}
	sox_write_config();
	gtk_widget_destroy(configure_win);
}

static void cancel_button_cb(GtkWidget *w, gpointer data) {
	gint i;

	gtk_widget_destroy(configure_win);

	for (i = 0; sox_configs[i].name; i++) {
		sox_configs[i].cancel();
	}

	sox_restart_effect = 1;
	sox_cfg = sox_cfg_old;
}

void sox_configure(void) {
	GtkWidget *label;
	GtkWidget *effect_box;
	gint i;

	if (configure_win) return;

	sox_read_config();
	sox_cfg_old = sox_cfg;

	configure_win = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_container_set_border_width(GTK_CONTAINER(configure_win), 10);
	gtk_window_set_title(GTK_WINDOW(configure_win), "SOX Effects Configuration");
	gtk_window_set_policy(GTK_WINDOW(configure_win), FALSE, FALSE, FALSE);
	gtk_window_set_position(GTK_WINDOW(configure_win), GTK_WIN_POS_MOUSE);
	gtk_signal_connect(GTK_OBJECT(configure_win), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &configure_win);

	win_box = gtk_vbox_new(FALSE, 5);

	effect = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(effect), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(win_box), effect, FALSE, FALSE, 0);
	gtk_widget_show(effect);

	for (i = 0; sox_configs[i].name; i++) {
		effect_box = gtk_vbox_new(FALSE, 5);
		sox_configs[i].render_opts(effect_box);
		gtk_widget_show(effect_box);

		label = gtk_label_new(sox_configs[i].name);
		gtk_widget_show(label);

		gtk_notebook_append_page(GTK_NOTEBOOK(effect), effect_box, label);
	}
	gtk_notebook_set_page(GTK_NOTEBOOK(effect), sox_cfg.effect_number);
	gtk_signal_connect(GTK_OBJECT(effect), "switch-page", sox_effect_cb, NULL);

	button_box = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(button_box), 5);
	gtk_box_pack_start(GTK_BOX(win_box), button_box, FALSE, FALSE, 0);

	ok_button = gtk_button_new_with_label("Ok");
	gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
		GTK_SIGNAL_FUNC(ok_button_cb), NULL);
	GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(button_box), ok_button, TRUE, TRUE, 0);
	gtk_widget_show(ok_button);

	cancel_button = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
		GTK_SIGNAL_FUNC(cancel_button_cb), NULL);
	GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(button_box), cancel_button, TRUE, TRUE, 0);
	gtk_widget_show(cancel_button);
	gtk_widget_show(button_box);

	gtk_container_add(GTK_CONTAINER(configure_win), win_box);
	gtk_widget_show(win_box);
	gtk_widget_show(configure_win);
	gtk_widget_grab_default(ok_button);
}
